@@ -25,7 +25,7 @@
 #include "Specific.hh"
 #include "ValidSchema.hh"
 
-#include <boost/bind.hpp>
+#include <boost/bind/bind.hpp>
 #include <cstdint>
 #include <functional>
 #include <stack>
@@ -34,7 +34,7 @@
 
 #include <boost/math/special_functions/fpclassify.hpp>
 #include <boost/random/mersenne_twister.hpp>
-#include <boost/test/included/unit_test_framework.hpp>
+#include <boost/test/included/unit_test.hpp>
 #include <boost/test/parameterized_test.hpp>
 #include <boost/test/unit_test.hpp>
 
@@ -68,7 +68,7 @@ static const unsigned int count = 10;
  * promote data type, convert from union to plain data type and vice versa,
  * insert or remove fields in records or reorder fields in a record.
  *
- * To test Json encoder and decoder, we use the same technqiue with only
+ * To test Json encoder and decoder, we use the same technique with only
  * one difference - we use JsonEncoder and JsonDecoder.
  *
  * We also use the same infrastructure to test GenericReader and GenericWriter.
@@ -160,7 +160,7 @@ static string randomString(size_t len) {
         if (c == '\0') {
             c = '\x7f';
         }
-        result.push_back(c);
+        result.push_back(static_cast<char>(c));
     }
     return result;
 }
@@ -169,7 +169,7 @@ static vector<uint8_t> randomBytes(size_t len) {
     vector<uint8_t> result;
     result.reserve(len);
     for (size_t i = 0; i < len; ++i) {
-        result.push_back(rnd());
+        result.push_back(static_cast<uint8_t>(rnd()));
     }
     return result;
 }
@@ -335,7 +335,7 @@ struct StackElement {
 };
 } // namespace
 
-static vector<string>::const_iterator skipCalls(Scanner &sc, Decoder &d,
+static vector<string>::const_iterator skipCalls(Scanner &sc, Decoder &,
                                                 vector<string>::const_iterator it, bool isArray) {
     char end = isArray ? ']' : '}';
     int level = 0;
@@ -364,7 +364,7 @@ static vector<string>::const_iterator skipCalls(Scanner &sc, Decoder &d,
             case 'K':
             case 'b':
             case 'f':
-            case 'e': ++it; // Fall through.
+            case 'e': ++it; [[fallthrough]];
             case 'c':
             case 'U':
                 sc.extractInt();
@@ -525,7 +525,7 @@ ValidSchema makeValidSchema(const char *schema) {
     istringstream iss(schema);
     ValidSchema vs;
     compileJsonSchema(iss, vs);
-    return ValidSchema(vs);
+    return vs;
 }
 
 void testEncoder(const EncoderPtr &e, const char *writerCalls,
@@ -594,7 +594,6 @@ struct TestData4 {
     const char *readerCalls;
     const char *readerValues[100];
     unsigned int depth;
-    size_t recordCount;
 };
 
 void appendSentinel(OutputStream &os) {
@@ -838,7 +837,7 @@ void testGenericResolving(const TestData3 &td) {
         GenericReader gr(wvs, rvs, d1);
         GenericDatum datum;
         gr.read(datum);
-        d1->drain();
+        gr.drain();
         assertSentinel(*in1);
 
         EncoderPtr e2 = CodecFactory::newEncoder(rvs);
@@ -963,6 +962,11 @@ static const TestData data[] = {
     {R"({"type":"map", "values": "boolean"})",
      "{c1sK5Bc2sK5BsK5B}", 2},
 
+    // Record with no fields
+    {"{\"type\":\"record\",\"name\":\"empty\",\"fields\":[]}",
+     "", 1},
+
+    // Single-field records
     {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
      "{\"name\":\"f\", \"type\":\"boolean\"}]}",
      "B", 1},
@@ -1002,6 +1006,16 @@ static const TestData data[] = {
      "{\"name\":\"f7\", \"type\":\"bytes\"}]}",
      "NBILFDS10b25", 1},
     // record of records
+    {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
+     "{\"name\":\"f1\",\"type\":\"boolean\"},"
+     "{\"name\":\"f2\", \"type\":{\"type\":\"record\","
+     "\"name\":\"inner\",\"fields\":[]}}]}",
+     "B", 1},
+    {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
+     "{\"name\":\"f1\",\"type\":\"boolean\"},"
+     "{\"name\":\"f2\", \"type\":{\"type\":\"array\","
+     "\"items\":\"r\"}}]}",
+     "B[]", 1},
     {"{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
      "{\"name\":\"f1\", \"type\":{\"type\":\"record\", "
      "\"name\":\"inner\", \"fields\":["
@@ -1264,249 +1278,558 @@ static const TestData3 data3[] = {
 
     {R"(["boolean", "int"])", "U1I", R"(["boolean", "long"])", "U1L", 1},
     {R"(["boolean", "int"])", "U1I", R"(["long", "boolean"])", "U0L", 1},
+
+    // Aliases
+    {R"({"type": "record", "name": "r", "fields": [
+        {"name": "f0", "type": "int"},
+        {"name": "f1", "type": "boolean"},
+        {"name": "f2", "type": "double"}]})",
+     "IBD",
+     R"({"type":"record", "name":"s", "aliases":["r"], "fields":[
+        {"name":"g0", "type":"int", "aliases":["f0"]},
+        {"name":"g1", "type":"boolean", "aliases":["f1"]},
+        {"name":"f2", "type":"double", "aliases":["g2"]}]})",
+     "IBD",
+     1},
+    {R"({"type": "record", "name": "r", "namespace": "n", "fields": [
+     {"name": "f0", "type": "int"}]})",
+     "I",
+     R"({"type": "record", "name": "s", "namespace": "n2", "aliases": ["t", "n.r"], "fields":[
+       {"name": "f0", "type": "int"}]})",
+     "I",
+     1},
+    {R"({"type": "enum", "name": "e", "symbols": ["a", "b"]})",
+     "e1",
+     R"({"type": "enum", "name": "f", "aliases": ["e"], "symbols":["a", "b", "c"]})",
+     "e1",
+     1},
+    {R"({"type": "enum", "name": "e", "namespace": "n", "symbols": ["a", "b"]})",
+     "e1",
+     R"({"type": "enum", "name": "f", "namespace": "n2", "aliases": ["g", "n.e"], "symbols": ["a", "b"]})",
+     "e1",
+     1},
+    {R"({"type": "fixed", "name": "f", "size": 8})",
+     "f8",
+     R"({"type": "fixed", "name": "g", "aliases": ["f"], "size": 8})",
+     "f8",
+     1},
+    {R"({"type": "fixed", "name": "f", "namespace": "n", "size": 8})",
+     "f8",
+     R"({"type": "fixed", "name": "g", "namespace": "n2", "aliases": ["h", "n.f"], "size": 8})",
+     "f8",
+     1},
+    {R"({"type": "record", "name": "r1", "fields": [
+     {"name": "f1", "type": ["null", {"type": "record", "name": "r2", "fields": [{"name": "f11", "type": "string"}]}]},
+     {"name": "f2", "type": {"type": "array", "items": "r2"}}
+     ]})",
+     "U0N[c3sS1sS2sS3]",
+     R"({"type": "record", "name": "r1", "fields": [
+     {"name": "f1", "type": [
+         "null",
+         {"type": "record", "name": "r2", "fields": [{"name": "f11", "type": "string"}]},
+         {"type": "record", "name": "r3", "fields": [
+             {"name": "g11", "type": {"type": "array", "items": {"type": "record", "name": "r31", "fields": [{"name": "g111", "type": "double"}]}}}
+         ]}
+     ]},
+     {"name": "f2", "type": {"type": "array", "items": "r2"}},
+     {"name": "f3", "type": {"type": "array", "items": "r3"}, "default": []}
+     ]})",
+     "U0N[c3sS1sS2sS3][]",
+     1},
+    {
+      R"({"name": "Project", "type": "record", "fields": [
+        { "name": "_types", "type": [
+            "null",
+            { "name": "Record1", "type": "record", "fields": [{ "name": "Record1_field1", "type": "string" }]}
+        ]},
+        { "name": "field1", "type": { "type": "array", "items": "Record1" } }
+      ]})",
+        "U0N[c3sS1sS2sS3]",
+      R"({"name": "Project", "type": "record", "fields": [
+        { "name": "_types", "type": [
+            "null",
+            { "name": "Record1", "type": "record", "fields": [{ "name": "Record1_field1", "type": "string" }]},
+            { "name": "Record3", "type": "record", "fields": [
+                { "name": "Record3_field1", "type": { "type": "array", "items": { "name": "Record2", "type": "record",
+                     "fields":[{ "name": "Record2_field1", "type": "double" }]}
+                }}
+            ]}
+        ]},
+        { "name": "field1", "type": { "type": "array", "items": "Record1" } },
+        { "name": "field2", "type": { "type": "array", "items": "Record3" }, "default": [] }
+      ]})",
+        "U0N[c3sS1sS2sS3][]",
+        1},
+    {
+        R"({"name": "Project", "type": "record", "fields": [
+        { "name": "_types", "type": [
+            "null",
+            { "name": "Record1", "type": "record", "fields": [{ "name": "Record1_field1", "type": "string" }]},
+            { "name": "Record3", "type": "record", "fields": [
+                { "name": "Record3_field1", "type": { "type": "array", "items": { "name": "Record2", "type": "record",
+                     "fields":[{ "name": "Record2_field1", "type": "double" }]}
+                }}
+            ]}
+        ]},
+        { "name": "field1", "type": { "type": "array", "items": "Record1" } },
+        { "name": "field2", "type": { "type": "array", "items": "Record3" }, "default": [] }
+      ]})",
+        "U0N[c3sS1sS2sS3][]",
+      R"({"name": "Project", "type": "record", "fields": [
+        { "name": "_types", "type": [
+            "null",
+            { "name": "Record1", "type": "record", "fields": [{ "name": "Record1_field1", "type": "string" }]}
+        ]},
+        { "name": "field1", "type": { "type": "array", "items": "Record1" } }
+      ]})",
+        "U0N[c3sS1sS2sS3]",
+        1},
 };
 
 static const TestData4 data4[] = {
     // Projection
-    {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"string\"},"
-     "{\"name\":\"f2\", \"type\":\"string\"},"
-     "{\"name\":\"f3\", \"type\":\"int\"}]}",
-     "S10S10IS10S10I",
-     {"s1", "s2", "100", "t1", "t2", "200", nullptr},
-     "{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"string\" },"
-     "{\"name\":\"f2\", \"type\":\"string\"}]}",
-     "RS10S10RS10S10",
-     {"s1", "s2", "t1", "t2", nullptr},
-     1,
-     2},
+    {
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f1", "type": "string"},
+                {"name": "f2", "type": "string"},
+                {"name": "f3", "type": "int"}
+            ]
+        })",
+        "S10S10IS10S10I",
+        {"s1", "s2", "100", "t1", "t2", "200", nullptr},
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f1", "type": "string"},
+                {"name": "f2", "type": "string"}
+            ]
+        })",
+        "RS10S10RS10S10",
+        {"s1", "s2", "t1", "t2", nullptr},
+        1},
 
     // Reordered fields
-    {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"int\"},"
-     "{\"name\":\"f2\", \"type\":\"string\"}]}",
-     "IS10",
-     {"10", "hello", nullptr},
-     "{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f2\", \"type\":\"string\" },"
-     "{\"name\":\"f1\", \"type\":\"long\"}]}",
-     "RLS10",
-     {"10", "hello", nullptr},
-     1,
-     1},
+    {
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f1", "type": "int"},
+                {"name": "f2", "type": "string"}
+            ]
+        })",
+        "IS10",
+        {"10", "hello", nullptr},
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f2", "type": "string" },
+                {"name": "f1", "type": "long"}
+            ]
+        })",
+        "RLS10",
+        {"10", "hello", nullptr},
+        1},
 
     // Default values
-    {R"({"type":"record","name":"r","fields":[]})", "", {nullptr}, "{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-                                                                   "{\"name\":\"f\", \"type\":\"int\", \"default\": 100}]}",
-     "RI",
-     {"100", nullptr},
-     1,
-     1},
+    {
+        R"({"type": "record", "name": "r", "fields": []})",
+        "",
+        {nullptr},
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [{"name": "f", "type": "int", "default": 100}]
+        })",
+        "RI",
+        {"100", nullptr},
+        1},
 
-    {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f2\", \"type\":\"int\"}]}",
+    {R"({"type": "record", "name": "r", "fields": [{"name": "f2", "type": "int"}]})",
      "I",
      {"10", nullptr},
-     "{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"int\", \"default\": 101},"
-     "{\"name\":\"f2\", \"type\":\"int\"}]}",
+     R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f1", "type": "int", "default": 101},
+                {"name": "f2", "type": "int"}
+            ]
+        })",
      "RII",
      {"10", "101", nullptr},
-     1,
      1},
 
-    {"{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner\",\"fields\":["
-     "{\"name\":\"f2\", \"type\":\"int\"}]}}, "
-     "{\"name\": \"g2\", \"type\": \"long\"}]}",
-     "IL",
-     {"10", "11", nullptr},
-     "{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"int\", \"default\": 101},"
-     "{\"name\":\"f2\", \"type\":\"int\"}]}}, "
-     "{\"name\": \"g2\", \"type\": \"long\"}]}}",
-     "RRIIL",
-     {"10", "101", "11", nullptr},
-     1,
-     1},
+    {
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "record",
+                        "name": "inner",
+                        "fields": [{"name": "f2", "type": "int"}]
+                    }
+                },
+                {"name": "g2", "type": "long"}
+            ]
+        })",
+        "IL",
+        {"10", "11", nullptr},
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "record",
+                        "name": "inner",
+                        "fields": [
+                            {
+                                "name": "f1",
+                                "type": "int",
+                                "default": 101
+                            },
+                            {"name": "f2", "type": "int"}
+                        ]
+                    }
+                },
+                {"name": "g2", "type": "long"}
+            ]
+        })",
+        "RRIIL",
+        {"10", "101", "11", nullptr},
+        1},
 
     // Default value for a record.
-    {"{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner1\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"long\" },"
-     "{\"name\":\"f2\", \"type\":\"int\"}] } }, "
-     "{\"name\": \"g2\", \"type\": \"long\"}]}",
-     "LIL",
-     {"10", "12", "13", nullptr},
-     "{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner1\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"long\" },"
-     "{\"name\":\"f2\", \"type\":\"int\"}] } }, "
-     "{\"name\": \"g2\", \"type\": \"long\"},"
-     "{\"name\": \"g3\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner2\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"long\" },"
-     "{\"name\":\"f2\", \"type\":\"int\"}] }, "
-     "\"default\": { \"f1\": 15, \"f2\": 101 } }] } ",
-     "RRLILRLI",
-     {"10", "12", "13", "15", "101", nullptr},
-     1,
-     1},
+    {
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "record",
+                        "name": "inner1",
+                        "fields": [
+                            {"name": "f1", "type": "long"},
+                            {"name": "f2", "type": "int"}
+                        ]
+                    }
+                },
+                {"name": "g2", "type": "long"}
+            ]
+        })",
+        "LIL",
+        {"10", "12", "13", nullptr},
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "record",
+                        "name": "inner1",
+                        "fields": [
+                            {"name": "f1", "type": "long"},
+                            {"name": "f2", "type": "int"}
+                        ]
+                    }
+                },
+                {"name": "g2", "type": "long"},
+                {
+                    "name": "g3",
+                    "type": {
+                        "type": "record",
+                        "name": "inner2",
+                        "fields": [
+                            {"name": "f1", "type": "long"},
+                            {"name": "f2", "type": "int"}
+                        ]
+                    },
+                    "default": {"f1": 15, "f2": 101}
+                }
+            ]
+        })",
+        "RRLILRLI",
+        {"10", "12", "13", "15", "101", nullptr},
+        1},
 
-    {"{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner1\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"long\" },"
-     "{\"name\":\"f2\", \"type\":\"int\"}] } }, "
-     "{\"name\": \"g2\", \"type\": \"long\"}]}",
-     "LIL",
-     {"10", "12", "13", nullptr},
-     "{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\", "
-     "\"type\":{\"type\":\"record\",\"name\":\"inner1\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"long\" },"
-     "{\"name\":\"f2\", \"type\":\"int\"}] } }, "
-     "{\"name\": \"g2\", \"type\": \"long\"},"
-     "{\"name\": \"g3\", "
-     "\"type\":\"inner1\", "
-     "\"default\": { \"f1\": 15, \"f2\": 101 } }] } ",
-     "RRLILRLI",
-     {"10", "12", "13", "15", "101", nullptr},
-     1,
-     1},
+    {
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "record",
+                        "name": "inner1",
+                        "fields": [
+                            {"name": "f1", "type": "long"},
+                            {"name": "f2", "type": "int"}
+                        ]
+                    }
+                },
+                {"name": "g2", "type": "long"}
+            ]
+        })",
+        "LIL",
+        {"10", "12", "13", nullptr},
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "record",
+                        "name": "inner1",
+                        "fields": [
+                            {"name": "f1", "type": "long"},
+                            {"name": "f2", "type": "int"}
+                        ]
+                    }
+                },
+                {"name": "g2", "type": "long"},
+                {
+                    "name": "g3",
+                    "type": "inner1",
+                    "default": {"f1": 15, "f2": 101}
+                }
+            ]
+        })",
+        "RRLILRLI",
+        {"10", "12", "13", "15", "101", nullptr},
+        1},
 
-    {R"({"type":"record","name":"r","fields":[]})", "", {nullptr}, "{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-                                                                   "{\"name\":\"f\", \"type\":{ \"type\": \"array\", \"items\": \"int\" },"
-                                                                   "\"default\": [100]}]}",
-     "[c1sI]",
-     {"100", nullptr},
-     1,
-     1},
+    // TODO mkmkme HERE
+    {
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": []
+        })",
+        "",
+        {nullptr},
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {
+                    "name": "f",
+                    "type": {"type": "array", "items": "int"},
+                    "default": [100]
+                }
+            ]
+        })",
+        "[c1sI]",
+        {"100", nullptr},
+        1},
 
-    {"{ \"type\": \"array\", \"items\": {\"type\":\"record\","
-     "\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f0\", \"type\": \"int\"}]} }",
-     "[c1sI]",
-     {"99", nullptr},
-     "{ \"type\": \"array\", \"items\": {\"type\":\"record\","
-     "\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f\", \"type\":\"int\", \"default\": 100}]} }",
-     "[Rc1sI]",
-     {"100", nullptr},
-     1,
-     1},
+    {
+        R"({
+            "type": "array",
+            "items": {
+                "type": "record",
+                "name": "r",
+                "fields": [{"name": "f0", "type": "int"}]
+            }
+        })",
+        "[c1sI]",
+        {"99", nullptr},
+        R"({
+            "type": "array",
+            "items": {
+                "type": "record",
+                "name": "r",
+                "fields": [{"name": "f", "type": "int", "default": 100}]
+            }
+        })",
+        "[Rc1sI]",
+        {"100", nullptr},
+        1},
 
     // Record of array of record with deleted field as last field
-    {"{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\","
-     "\"type\":{\"type\":\"array\",\"items\":{"
-     "\"name\":\"item\",\"type\":\"record\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"int\"},"
-     "{\"name\":\"f2\", \"type\": \"long\", \"default\": 0}]}}}]}",
-     "[c1sIL]",
-     {"10", "11", nullptr},
-     "{\"type\":\"record\",\"name\":\"outer\",\"fields\":["
-     "{\"name\": \"g1\","
-     "\"type\":{\"type\":\"array\",\"items\":{"
-     "\"name\":\"item\",\"type\":\"record\",\"fields\":["
-     "{\"name\":\"f1\", \"type\":\"int\"}]}}}]}",
-     "R[c1sI]",
-     {"10", nullptr},
-     2,
-     1},
+    {
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields":[
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "array",
+                        "items": {
+                            "name": "item",
+                            "type": "record",
+                            "fields": [
+                                {"name": "f1", "type": "int"},
+                                {"name": "f2", "type": "long", "default": 0}
+                            ]
+                        }
+                    }
+                }
+            ]
+        })",
+        "[c1sIL]",
+        {"10", "11", nullptr},
+        R"({
+            "type": "record",
+            "name": "outer",
+            "fields": [
+                {
+                    "name": "g1",
+                    "type": {
+                        "type": "array",
+                        "items": {
+                            "name": "item",
+                            "type": "record",
+                            "fields": [{"name": "f1", "type": "int"}]
+                        }
+                    }
+                }
+            ]
+        })",
+        "R[c1sI]",
+        {"10", nullptr},
+        2},
 
     // Enum resolution
-    {R"({"type":"enum","name":"e","symbols":["x","y","z"]})",
-     "e2",
-     {nullptr},
-     R"({"type":"enum","name":"e","symbols":[ "y", "z" ]})",
-     "e1",
-     {nullptr},
-     1,
-     1},
+    {
+        R"({"type":"enum","name":"e","symbols":["x","y","z"]})",
+        "e2",
+        {nullptr},
+        R"({"type": "enum", "name": "e", "symbols": ["y", "z"]})",
+        "e1",
+        {nullptr},
+        1},
 
-    {R"({"type":"enum","name":"e","symbols":[ "x", "y" ]})",
+    {R"({"type": "enum", "name": "e", "symbols": ["x", "y"]})",
      "e1",
      {nullptr},
-     R"({"type":"enum","name":"e","symbols":[ "y", "z" ]})",
+     R"({"type": "enum", "name": "e", "symbols": ["y", "z"]})",
      "e0",
      {nullptr},
-     1,
      1},
 
     // Union
-    {"\"int\"", "I", {"100", nullptr}, R"([ "long", "int"])", "U1I", {"100", nullptr}, 1, 1},
+    {
+        R"("int")",
+        "I",
+        {"100", nullptr},
+        R"(["long", "int"])",
+        "U1I",
+        {"100", nullptr},
+        1},
 
-    {R"([ "long", "int"])", "U1I", {"100", nullptr}, "\"int\"", "I", {"100", nullptr}, 1, 1},
+    {R"(["long", "int"])",
+     "U1I",
+     {"100", nullptr},
+     R"("int")",
+     "I",
+     {"100", nullptr},
+     1},
 
     // Arrray of unions
-    {R"({"type":"array", "items":[ "long", "int"]})",
-     "[c2sU1IsU1I]",
-     {"100", "100", nullptr},
-     R"({"type":"array", "items": "int"})",
-     "[c2sIsI]",
-     {"100", "100", nullptr},
-     2,
-     1},
+    {
+        R"({"type": "array", "items": ["long", "int"]})",
+        "[c2sU1IsU1I]",
+        {"100", "100", nullptr},
+        R"({"type":"array", "items": "int"})",
+        "[c2sIsI]",
+        {"100", "100", nullptr},
+        2},
 
     // Map of unions
-    {R"({"type":"map", "values":[ "long", "int"]})",
-     "{c2sS10U1IsS10U1I}",
-     {"k1", "100", "k2", "100", nullptr},
-     R"({"type":"map", "values": "int"})",
-     "{c2sS10IsS10I}",
-     {"k1", "100", "k2", "100", nullptr},
-     2,
-     1},
+    {
+        R"({"type": "map", "values": ["long", "int"]})",
+        "{c2sS10U1IsS10U1I}",
+        {"k1", "100", "k2", "100", nullptr},
+        R"({"type":"map", "values": "int"})",
+        "{c2sS10IsS10I}",
+        {"k1", "100", "k2", "100", nullptr},
+        2},
 
     // Union + promotion
-    {"\"int\"", "I", {"100", nullptr}, R"([ "long", "string"])", "U0L", {"100", nullptr}, 1, 1},
+    {
+        R"("int")",
+        "I",
+        {"100", nullptr},
+        R"(["long", "string"])",
+        "U0L",
+        {"100", nullptr},
+        1},
 
-    {R"([ "int", "string"])", "U0I", {"100", nullptr}, "\"long\"", "L", {"100", nullptr}, 1, 1},
+    {R"(["int", "string"])",
+     "U0I",
+     {"100", nullptr},
+     R"("long")",
+     "L",
+     {"100", nullptr},
+     1},
 
     // Record where union field is skipped.
-    {"{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f0\", \"type\":\"boolean\"},"
-     "{\"name\":\"f1\", \"type\":\"int\"},"
-     "{\"name\":\"f2\", \"type\":[\"int\", \"long\"]},"
-     "{\"name\":\"f3\", \"type\":\"float\"}"
-     "]}",
-     "BIU0IF",
-     {"1", "100", "121", "10.75", nullptr},
-     "{\"type\":\"record\",\"name\":\"r\",\"fields\":["
-     "{\"name\":\"f0\", \"type\":\"boolean\"},"
-     "{\"name\":\"f1\", \"type\":\"long\"},"
-     "{\"name\":\"f3\", \"type\":\"double\"}]}",
-     "BLD",
-     {"1", "100", "10.75", nullptr},
-     1,
-     1},
+    {
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f0", "type": "boolean"},
+                {"name": "f1", "type": "int"},
+                {"name": "f2", "type": ["int", "long"]},
+                {"name": "f3", "type": "float"}
+            ]
+        })",
+        "BIU0IF",
+        {"1", "100", "121", "10.75", nullptr},
+        R"({
+            "type": "record",
+            "name": "r",
+            "fields": [
+                {"name": "f0", "type": "boolean"},
+                {"name": "f1", "type": "long"},
+                {"name": "f3", "type": "double"}
+            ]
+        })",
+        "BLD",
+        {"1", "100", "10.75", nullptr},
+        1},
 };
 
 static const TestData4 data4BinaryOnly[] = {
     // Arrray of unions
-    {R"({"type":"array", "items":[ "long", "int"]})",
-     "[c1sU1Ic1sU1I]",
-     {"100", "100", nullptr},
-     R"({"type":"array", "items": "int"})",
-     "[c1sIc1sI]",
-     {"100", "100", nullptr},
-     2},
+    {
+        R"({
+            "type":"array",
+            "items": ["long", "int"]
+        })",
+        "[c1sU1Ic1sU1I]",
+        {"100", "100", nullptr},
+        R"({"type":"array", "items": "int"})",
+        "[c1sIc1sI]",
+        {"100", "100", nullptr},
+        2},
 
     // Map of unions
-    {R"({"type":"map", "values":[ "long", "int"]})",
-     "{c1sS10U1Ic1sS10U1I}",
-     {"k1", "100", "k2", "100", nullptr},
-     R"({"type":"map", "values": "int"})",
-     "{c1sS10Ic1sS10I}",
-     {"k1", "100", "k2", "100", nullptr},
-     2},
+    {
+        R"({"type":"map", "values":[ "long", "int"]})",
+        "{c1sS10U1Ic1sS10U1I}",
+        {"k1", "100", "k2", "100", nullptr},
+        R"({"type":"map", "values": "int"})",
+        "{c1sS10Ic1sS10I}",
+        {"k1", "100", "k2", "100", nullptr},
+        2},
 };
 
 #define COUNTOF(x) sizeof(x) / sizeof(x[0])
@@ -1524,13 +1847,13 @@ Test testWithData(const Test &test, const Data &) {
         testWithData(&testFunc<Factory>, data), data, data + COUNTOF(data)))
 
 struct BinaryEncoderFactory {
-    static EncoderPtr newEncoder(const ValidSchema &schema) {
+    static EncoderPtr newEncoder(const ValidSchema &) {
         return binaryEncoder();
     }
 };
 
 struct BinaryDecoderFactory {
-    static DecoderPtr newDecoder(const ValidSchema &schema) {
+    static DecoderPtr newDecoder(const ValidSchema &) {
         return binaryDecoder();
     }
 };
@@ -1791,7 +2114,7 @@ static void testByteCount() {
 } // namespace avro
 
 boost::unit_test::test_suite *
-init_unit_test_suite(int argc, char *argv[]) {
+init_unit_test_suite(int, char *[]) {
     using namespace boost::unit_test;
 
     auto *ts = BOOST_TEST_SUITE("Avro C++ unit tests for codecs");
