@@ -34,7 +34,7 @@ import org.junit.jupiter.api.Test;
 
 public class TestUtf8 {
   @Test
-  public void testByteConstructor() throws Exception {
+  void byteConstructor() throws Exception {
     byte[] bs = "Foo".getBytes(StandardCharsets.UTF_8);
     Utf8 u = new Utf8(bs);
     assertEquals(bs.length, u.getByteLength());
@@ -44,7 +44,7 @@ public class TestUtf8 {
   }
 
   @Test
-  public void testArrayReusedWhenLargerThanRequestedSize() {
+  void arrayReusedWhenLargerThanRequestedSize() {
     byte[] bs = "55555".getBytes(StandardCharsets.UTF_8);
     Utf8 u = new Utf8(bs);
     assertEquals(5, u.getByteLength());
@@ -58,7 +58,7 @@ public class TestUtf8 {
   }
 
   @Test
-  public void testHashCodeReused() {
+  void hashCodeReused() {
     assertEquals(97, new Utf8("a").hashCode());
     assertEquals(3904, new Utf8("zz").hashCode());
     assertEquals(122, new Utf8("z").hashCode());
@@ -119,7 +119,7 @@ public class TestUtf8 {
   }
 
   @Test
-  public void testSerialization() throws IOException, ClassNotFoundException {
+  void serialization() throws IOException, ClassNotFoundException {
     try (ByteArrayOutputStream bos = new ByteArrayOutputStream();
         ObjectOutputStream oos = new ObjectOutputStream(bos)) {
 
